//----------------------------------------------------------------------------//
//                                                                            //
//                           MB7707 board examples                            //
//                                                                            //
//                                Demo program                                //
//                         calculate factorial of 10.                         //
//                           Host part of program.                            //
//                                                                            //
//                  2017 (c) RC Module Inc., Moscow, Russia                   //
//                                                                            //
//----------------------------------------------------------------------------//

#include <iostream>
#include <stdlib.h>
#include <windows.h>

#include "mb7707load.h"

using namespace std;

const unsigned int CORE_NUMBER = 0;
unsigned char MAC_ADDRESS[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

const char *NM_PART_FILE_NAME = "nm_part.abs";

int main()
{
	PL_Board *board;
	PL_Access *access;
	PL_Word Status, NM_part_return_value;
	int ret = 0;

	if (PL_GetBoardDesc(MAC_ADDRESS, &board) != PL_OK)
	{
		cout << "ERROR: Failed open board!\n";
		exit(1);
	}

	if (PL_GetAccess(board, CORE_NUMBER, &access) != PL_OK)
	{
		cout << "ERROR: Failed access core " << CORE_NUMBER << "!\n";
		exit(1);
	}

	if (PL_LoadInitCode(access) != PL_OK)
	{
		cout << "ERROR: Failed load init code!\n";
		exit(1);
	}

	if (PL_LoadProgramFile(access, NM_PART_FILE_NAME) != PL_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Failed load program!\n";
		exit(1);
	}

	while (1)
	{
		if (PL_GetStatus(access, &Status) != PL_OK)
		{
			cout << "ERROR: Failed get status!\n";
			exit(1);
		}

		if (Status & PROGRAM_FINISHED)
			break;

		Sleep(1);
	}

	if (PL_GetResult(access, &NM_part_return_value) != PL_OK)
	{
		cout << "ERROR: Failed get NM program return value!";
		exit(1);
	}

	if (NM_part_return_value == 3628800)
		cout << "10! = " << NM_part_return_value << endl;
	else
	{
		cout << "ERROR: 10! <> " << NM_part_return_value << endl;
		ret = 1;
	}

	if (PL_CloseAccess(access) != PL_OK)
	{
		cout << "ERROR: Failed close core!\n";
		exit(1);
	}

	if (PL_CloseBoardDesc(board) != PL_OK)
	{
		cout << "ERROR: Failed close board!\n";
		exit(1);
	}

	if (!ret)
		cout << "Test Ok.\n";

	return ret;
}
