//----------------------------------------------------------------------------//
//                                                                            //
//                           MB7707 board examples                            //
//                                                                            //
//                                Demo program                                //
//                  that exchange arrays with NMC3 on board.                  //
//                           Host part of program.                            //
//                                                                            //
//                  2017 (c) RC Module Inc., Moscow, Russia                   //
//                                                                            //
//----------------------------------------------------------------------------//

#include <iostream>
#include <stdlib.h>

#include "mb7707load.h"

using namespace std;

const unsigned int CORE_NUMBER = 0;
unsigned char MAC_ADDRESS[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

const char *NM_PART_FILE_NAME = "nm_part.abs";

int main()
{
	PL_Board *board;
	PL_Access *access;

	if (PL_GetBoardDesc(MAC_ADDRESS, &board) != PL_OK)
	{
		cout << "ERROR: Failed open board!\n";
		exit(1);
	}

	if (PL_GetAccess(board, CORE_NUMBER, &access) != PL_OK)
	{
		cout << "ERROR: Failed access core " << CORE_NUMBER << "!\n";
		exit(1);
	}

	if (PL_LoadInitCode(access) != PL_OK)
	{
		cout << "ERROR: Failed load init code!\n";
		exit(1);
	}

	if (PL_LoadProgramFile(access, NM_PART_FILE_NAME) != PL_OK)
	{
		cout << NM_PART_FILE_NAME << " :: ERROR: Failed load program!\n";
		exit(1);
	}

	PL_Word Length = 0;
	PL_Word *Buffer = 0;
	PL_Word AddrLoad = 0;
	PL_Word AddrStore = 0;
	PL_Word syncValue = 0;

	// Receive input array address
	if (PL_SyncArray(access,
		0, NULL, NULL,
		&syncValue, &AddrLoad, &Length)
		!= PL_OK)
	{
		cout << "Error in sync 1.\n";
		return 2;
	}

	if (syncValue != 1)
	{
		cout << "Error value in sync 1.\n";
		return 2;
	}

	Buffer = new PL_Word[Length];

	for (int i = 1; i < 31; i++)
	{
		// Sync for receive array and address of output array
		if (PL_SyncArray(access,
			1, NULL, NULL,
			&syncValue, &AddrStore, NULL)
			!= PL_OK)
		{
			cout << "Error in sync 2.\n";
			return 2;
		}

		if (syncValue != 2)
		{
			cout << "Error value in sync 2.\n";
			return 2;
		}

		// Receive array
		if (PL_ReadMemBlock(access, Buffer, AddrLoad, Length) != PL_OK)
		{
			cout << "Error in read.\n";
			return 1;
		}

		cout << "Verify number " << i << ".\n";

		for (int j = 0; j < Length; j++)
		{
			if (Buffer[j] != i)
			{
				cout << "Bad buffer[" << j << "] on step " << i << ".\n";
				return 3;
			}

			// New fill of array
			Buffer[j] = i + 1;
		}

		if (PL_WriteMemBlock(access, Buffer, AddrStore, Length) != PL_OK)
		{
			cout << "Error in write.\n";
			return 1;
		}

		// Sync for send array
		if (PL_Sync(access,
			2, &syncValue)
			!= PL_OK)
		{
			cout << "Error in sync 3.\n";
			return 2;
		}

		if (syncValue != 3)
		{
			cout << "Error value in sync 3.\n";
			return 2;
		}
	}

	// Sync for end program
	if (PL_Sync(access,
		17, NULL)
		!= PL_OK)
	{
		cout << "Error value in sync 4.\n";
		return 2;
	}

	delete Buffer;

	if (PL_CloseAccess(access) != PL_OK)
	{
		cout << "ERROR: Failed close core!\n";
		exit(1);
	}

	if (PL_CloseBoardDesc(board) != PL_OK)
	{
		cout << "ERROR: Failed close board!\n";
		exit(1);
	}

	cout << "Test Ok.\n";
	return 0;
}
